﻿define("epi-ecf-ui/contentediting/editors/model/CommerceMediaCollectionEditorModel", [
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/Deferred",
    // dojox
    "dojox/math/stats",
    // epi
    "epi/shell/command/DelegateCommand",
    "epi-cms/contentediting/editors/model/CollectionEditorModel",
    "epi-cms/dgrid/formatters",
    // Resources
    "epi/i18n!epi/cms/nls/commerce.contentediting.editors.commercemediacollectioneditor"
], function (
    array,
    declare,
    Deferred,
    // dojox
    mathStats,
     // epi
    DelegateCommand,
    CollectionEditorModel,
    formatters,
    // Resources
    res
) {
    return declare([CollectionEditorModel], {

        addItem: function (item, refItem, before) {
            // summary:
            //      Add new item.
            // item: Object
            //      The item raw data
            // refItem: Object
            //      The reference item (item model instance)
            // before: Boolean
            //      Indicates that the new item should be added before the reference item.
            // tags:
            //      public override
            item.groupName = item.groupName || this.defaultAssetGroupName;
            var items = this.get("items");
            if (!items || items.length < 1) {
                return this.inherited(arguments);
            }

            // check if adding item is existed, to move it to top
            // instead of adding duplicated item.
            var itemIdentifier = item.permanentUrl || item.assetKey;
            var existingItem;
            array.some(items, function (oldItem) {
                if (oldItem && oldItem.assetKey === itemIdentifier) {
                    existingItem = oldItem;
                    return true;
                }
            });
            if (existingItem) {
                var def = new Deferred();
                this.moveItem(existingItem, refItem || existingItem, before || true);
                def.resolve();
                return def;
            } else {
                var sortOrders = array.map(items, function (item, index) {
                    return item.sortOrder;
                });

                var maxSortOrder = mathStats.max(sortOrders);
                item.sortOrder = maxSortOrder + 1;
                return this.inherited(arguments);
            }
        },

        saveItem: function (item, index) {
            // summary:
            //      Save an item.
            // item: Object
            //      The item raw data
            // index: Number
            //      The item index
            // tags:
            //      public override

            var items = this.get("items");
            if (!items || items.length < 1) {
                return this.inherited(arguments);
            }

            // Don't save if there already is an item with this key.
            var itemExists;
            var currentIndex = 0;
            var itemIdentifier = item.permanentUrl || item.assetKey;
            array.some(items, function (oldItem) {
                if (oldItem && oldItem.assetKey === itemIdentifier && currentIndex !== index) {
                    itemExists = true;
                    return true;
                }
                currentIndex++;
            });

            if (!itemExists) {
                return this.inherited(arguments);
            }
        },

        moveItem: function (item, refItem, before) {
            // summary:
            //      Move an item.
            // item: Object
            //      The item model instance
            // refItem: Object
            //      The reference item (item model instance)
            // before: Boolean
            //      Indicates that the new item should be moved before the reference item.
            // tags:
            //      public override

            var itemSortOrder = item.sortOrder;
            // Update sort order of moving item.
            item.sortOrder = refItem.sortOrder;
            refItem.sortOrder = itemSortOrder;

            // Because the base is call to _onItemsChanged to update value for item of the model
            // so we have to update the 'sortOrder' property before call to 'moveItem' of the base.
            this.inherited(arguments);
        },

        getListCommands: function (availableCommands) {
            // summary:
            //      get list commands from avaible commands
            // tags:
            //      public override
            return [
                new DelegateCommand({
                    name: "add",
                    label: res.addlabel,
                    iconClass: "epi-iconPlus",
                    canExecute: true,
                    isAvailable: true,
                    delegate: this.addItemDelegate.bind(this)
                })
            ];
        },

        generateFormatters: function (columnDefinitions) {
            // summary:
            //      Generate formatters for the specified column definitions.
            // columnDefinitions:
            //      The definition for the columns that should get the generated formatters.
            // tags:
            //      public override

            var thumbnailFormatter = function (value) {
                return [formatters.thumbnail(value)];
            };

            for (var columnName in columnDefinitions) {
                if (columnDefinitions[columnName].isThumbnail) {
                    columnDefinitions[columnName].formatter = thumbnailFormatter;

                    break;
                }
            }
            
            return columnDefinitions;
        }
    });
});